package mvd.jester.tests;

import java.lang.reflect.Constructor;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import mvd.jester.model.SystemSetup;

/**
 * TestEnvironment
 */
public class TestEnvironment {

    private final long numberOfTaskSets;
    private final SystemSetup systemSetup;
    private final SystemSetup.Builder builder;
    private final Set<Constructor<? extends AbstractTest>> abstractTests;

    public TestEnvironment(SystemSetup.Builder builder, long numberOfTaskSets) {
        this.numberOfTaskSets = numberOfTaskSets;
        abstractTests = new HashSet<>();

        this.builder = builder;
        this.systemSetup = builder.build();
    }

    public TestEnvironment registerTestInterface(Class<? extends AbstractTest> abstractTest) {
        try {
            abstractTests.add(abstractTest.getConstructor(SystemSetup.class));
        } catch (Exception e) {
            System.out.println("Missing constructor!");
        }

        return this;
    }

    public void runTests() {
        Map<AbstractTest, Long> testCases = new HashMap<>();

        for (Constructor<? extends AbstractTest> t : abstractTests) {
            try {
                testCases.put(t.newInstance(this.systemSetup), (long) 0);
            } catch (Exception e) {
                System.out.println("Ahhh SHIT!");
            }
        }


        long checkedTasksets = 0;

        while (checkedTasksets < numberOfTaskSets) {
            builder.rebuild(this.systemSetup);

            double utilization = this.systemSetup.getUtilization();

            while (utilization <= this.systemSetup.getNumberOfProcessors()
                    && checkedTasksets < numberOfTaskSets) {
                checkedTasksets++;

                for (AbstractTest t : testCases.keySet()) {
                    if (t.runSchedulabilityCheck()) {
                        testCases.computeIfPresent(t, (k, v) -> v + 1);
                    }
                }
                builder.addTask(systemSetup);

                utilization = this.systemSetup.getUtilization();
            }
        }


        for (AbstractTest t : testCases.keySet()) {
            System.out.println("Testcase " + t.getName() + " scheduled " + testCases.get(t) + " of "
                    + numberOfTaskSets + " tasks!");
        }


    }
}
